﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.EntityFrameworkCore;

namespace Practice.Models;

public partial class SigurDatabaseContext : DbContext
{
    public SigurDatabaseContext()
    {
    }

    public SigurDatabaseContext(DbContextOptions<SigurDatabaseContext> options)
        : base(options)
    {
    }

    public virtual DbSet<AppealInfo> AppealInfos { get; set; }

    public virtual DbSet<AppealsTaken> AppealsTakens { get; set; }

    public virtual DbSet<BaseModule> BaseModules { get; set; }

    public virtual DbSet<CompanyInfo> CompanyInfos { get; set; }

    public virtual DbSet<ComplitedAppeal> ComplitedAppeals { get; set; }

    public virtual DbSet<Integration> Integrations { get; set; }

    public virtual DbSet<MainInfo> MainInfos { get; set; }

    public virtual DbSet<ReadyDecicion> ReadyDecicions { get; set; }

    public virtual DbSet<Staff> Staff { get; set; }

    public virtual DbSet<StatusInfo> StatusInfos { get; set; }

    protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        => optionsBuilder.UseNpgsql("Host=edu.pg.ngknn.ru;Port=5442;Database=sigur_database;Username=31P;Password=12345");

    protected override void OnModelCreating(ModelBuilder modelBuilder)
    {
        modelBuilder.Entity<AppealInfo>(entity =>
        {
            entity.HasKey(e => e.AppealNumber).HasName("appeal_info_pkey");

            entity.ToTable("appeal_info");

            entity.Property(e => e.AppealNumber).HasColumnName("appeal_number");
            entity.Property(e => e.DescriptionAppeal)
                .HasMaxLength(150)
                .HasColumnName("description_appeal");
        });

        modelBuilder.Entity<AppealsTaken>(entity =>
        {
            entity
                .HasKey(e => new { e.StaffLogin, e.AppealNumber }).HasName("appeals_taken_pkey");
                
            entity.ToTable("appeals_taken");

            entity.Property(e => e.AppealNumber).HasColumnName("appeal_number");
            entity.Property(e => e.StaffLogin)
                .HasMaxLength(50)
                .HasColumnName("staff_login");

            entity.HasOne(d => d.AppealNumberNavigation).WithMany()
                .HasForeignKey(d => d.AppealNumber)
                .HasConstraintName("appeals_taken_appeal_number_fkey");

            entity.HasOne(d => d.StaffLoginNavigation).WithMany()
                .HasForeignKey(d => d.StaffLogin)
                .HasConstraintName("appeals_taken_staff_login_fkey");
        });

        modelBuilder.Entity<BaseModule>(entity =>
        {
            entity.HasKey(e => e.VariantBaseModule).HasName("base_module_pkey");

            entity.ToTable("base_module");

            entity.Property(e => e.VariantBaseModule)
                .ValueGeneratedNever()
                .HasColumnName("variant_base_module");
            entity.Property(e => e.CountMembers).HasColumnName("count_members");
            entity.Property(e => e.DescriptionModule)
                .HasMaxLength(30)
                .HasColumnName("description_module");
        });

        modelBuilder.Entity<CompanyInfo>(entity =>
        {
            entity.HasKey(e => e.CompanyName).HasName("company_info_pkey");

            entity.ToTable("company_info");

            entity.Property(e => e.CompanyName)
                .HasMaxLength(50)
                .HasColumnName("company_name");
            entity.Property(e => e.CountControllers).HasColumnName("count_controllers");
            entity.Property(e => e.CountReaders).HasColumnName("count_readers");
            entity.Property(e => e.IntegrationId).HasColumnName("integration_id");
            entity.Property(e => e.IntegrationsEss).HasColumnName("integrations_ess");

            entity.HasOne(d => d.Integration).WithMany(p => p.CompanyInfos)
                .HasForeignKey(d => d.IntegrationId)
                .HasConstraintName("company_info_integration_id_fkey");
        });

        modelBuilder.Entity<ComplitedAppeal>(entity =>
        {
            entity.HasKey(e => e.NumberAppeal);
                
            entity.ToTable("complited_appeals");

            entity.HasIndex(e => e.NumberAppeal, "complited_appeals_number_appeal_key").IsUnique();

            entity.Property(e => e.AppealDecision)
                .HasMaxLength(10485760)
                .HasColumnName("appeal_decision");
            entity.Property(e => e.AppealProblem)
                .HasMaxLength(10485760)
                .HasColumnName("appeal_problem");
            entity.Property(e => e.NumberAppeal).HasColumnName("number_appeal");

            entity.HasOne(d => d.NumberAppealNavigation).WithOne()
                .HasForeignKey<ComplitedAppeal>(d => d.NumberAppeal)
                .OnDelete(DeleteBehavior.ClientSetNull)
                .HasConstraintName("complited_appeals_number_appeal_fkey");
        });

        modelBuilder.Entity<Integration>(entity =>
        {
            entity.HasKey(e => e.IntegrationId).HasName("integrations_pkey");

            entity.ToTable("integrations");

            entity.Property(e => e.IntegrationId).HasColumnName("integration_id");
            entity.Property(e => e.Awt).HasColumnName("awt");
            entity.Property(e => e.Cp).HasColumnName("cp");
            entity.Property(e => e.Etsc).HasColumnName("etsc");
            entity.Property(e => e.Gdp).HasColumnName("gdp");
            entity.Property(e => e.MtOffline).HasColumnName("mt_offline");
            entity.Property(e => e.Onec).HasColumnName("onec");
            entity.Property(e => e.Pa).HasColumnName("pa");
            entity.Property(e => e.Ps).HasColumnName("ps");
            entity.Property(e => e.ReadyDecicionNumber).HasColumnName("ready_decicion_number");
            entity.Property(e => e.Rod).HasColumnName("rod");
            entity.Property(e => e.Roe).HasColumnName("roe");
            entity.Property(e => e.Sod).HasColumnName("sod");
            entity.Property(e => e.VariantBaseModule).HasColumnName("variant_base_module");

            entity.HasOne(d => d.ReadyDecicionNumberNavigation).WithMany(p => p.Integrations)
                .HasForeignKey(d => d.ReadyDecicionNumber)
                .HasConstraintName("integrations_ready_decicion_number_fkey");

            entity.HasOne(d => d.VariantBaseModuleNavigation).WithMany(p => p.Integrations)
                .HasForeignKey(d => d.VariantBaseModule)
                .HasConstraintName("integrations_variant_base_module_fkey");
        });

        modelBuilder.Entity<MainInfo>(entity =>
        {
            entity.HasKey(e => e.AppealNumber).HasName("main_info_pkey");

            entity.ToTable("main_info");

            entity.Property(e => e.AppealNumber).HasColumnName("appeal_number");
            entity.Property(e => e.AppealDate).HasColumnName("appeal_date");
            entity.Property(e => e.AppealReason).HasColumnName("appeal_reason");
            entity.Property(e => e.CompanyName)
                .HasMaxLength(50)
                .HasColumnName("company_name");
            entity.Property(e => e.ContactNumber)
                .HasMaxLength(30)
                .HasColumnName("contact_number");
            entity.Property(e => e.Email)
                .HasMaxLength(60)
                .HasColumnName("email");
            entity.Property(e => e.Fcs)
                .HasMaxLength(60)
                .HasColumnName("fcs");
            entity.Property(e => e.StatusNumber).HasColumnName("status_number");

            entity.HasOne(d => d.AppealReasonNavigation).WithMany(p => p.MainInfos)
                .HasForeignKey(d => d.AppealReason)
                .HasConstraintName("main_info_appeal_reason_fkey");

            entity.HasOne(d => d.CompanyNameNavigation).WithMany(p => p.MainInfos)
                .HasForeignKey(d => d.CompanyName)
                .HasConstraintName("main_info_company_name_fkey");

            entity.HasOne(d => d.StatusNumberNavigation).WithMany(p => p.MainInfos)
                .HasForeignKey(d => d.StatusNumber)
                .HasConstraintName("main_info_status_number_fkey");
        });

        modelBuilder.Entity<ReadyDecicion>(entity =>
        {
            entity.HasKey(e => e.ReadyDecicionNumber).HasName("ready_decicion_pkey");

            entity.ToTable("ready_decicion");

            entity.Property(e => e.ReadyDecicionNumber)
                .ValueGeneratedNever()
                .HasColumnName("ready_decicion_number");
            entity.Property(e => e.DecicionName)
                .HasMaxLength(30)
                .HasColumnName("decicion_name");
        });

        modelBuilder.Entity<Staff>(entity =>
        {
            entity.HasKey(e => e.StaffLogin).HasName("staff_pkey");

            entity.ToTable("staff");

            entity.Property(e => e.StaffLogin)
                .HasMaxLength(50)
                .HasColumnName("staff_login");
            entity.Property(e => e.StaffFcs)
                .HasMaxLength(100)
                .HasColumnName("staff_fcs");
            entity.Property(e => e.StaffPassword)
                .HasMaxLength(100)
                .HasColumnName("staff_password");
        });

        modelBuilder.Entity<StatusInfo>(entity =>
        {
            entity.HasKey(e => e.StatusNumber).HasName("status_info_pkey");

            entity.ToTable("status_info");

            entity.Property(e => e.StatusNumber).HasColumnName("status_number");
            entity.Property(e => e.Condition)
                .HasMaxLength(20)
                .HasColumnName("condition");
        });

        OnModelCreatingPartial(modelBuilder);
    }

    partial void OnModelCreatingPartial(ModelBuilder modelBuilder);
}
