using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using DynamicData;
using Microsoft.EntityFrameworkCore.Storage;
using Practice.Models;
using ReactiveUI;

namespace Practice.ViewModels
{
	public class ClientViewModel : ReactiveObject
    {
        public ClientViewModel(Staff currentUser)
        {
            CurrentUser = currentUser;
            var clients = DBManager.GetCompanies();
            foreach (var client in clients)
            {
                ClientList.Add(new ClientListTemplate(client));
            }
            SelectedClient = ClientList[0];
        }

        private ObservableCollection<ClientParamTemplate> paramList;
        private ObservableCollection<ClientParamTemplate> moduleList;
        private ObservableCollection<TicketListTemplate> ticketList;
        private ClientListTemplate selectedClient;
        private string companyName;

        public Staff CurrentUser { get; set; }
        public ObservableCollection<ClientListTemplate> ClientList { get; set; } = [];
        public string CompanyName { get => companyName; set => this.RaiseAndSetIfChanged(ref companyName, value); }
        public ClientListTemplate SelectedClient { get => selectedClient; set => OnSelectedClientChange(ref selectedClient, value); }
        public ObservableCollection<ClientParamTemplate> ParamList { get => paramList; set => this.RaiseAndSetIfChanged(ref paramList, value); }
        public ObservableCollection<ClientParamTemplate> ModuleList { get => moduleList; set => this.RaiseAndSetIfChanged(ref moduleList, value); }
        public ObservableCollection<TicketListTemplate> TicketList { get => ticketList; set => this.RaiseAndSetIfChanged(ref ticketList, value); }

        private void OnSelectedClientChange(ref ClientListTemplate backingField, ClientListTemplate value) 
        {
            backingField = value;
            CompanyName = value.CompanyName;
            ObservableCollection<ClientParamTemplate> newParamList =
            [
                new(" ", value.CompanyName),
                new(" ", value.CountTickets.ToString()),
                new(" ", value.CountControllers.ToString()),
                new("", value.License)
            ];
            ParamList = newParamList;
            ModuleList = value.ModuleList;
            TicketList = value.TicketList;
        }
    }

    public class ClientListTemplate
    {
        public ClientListTemplate(CompanyInfo client) 
        {
            CompanyName = client.CompanyName;
            CountTickets = client.MainInfos.Count;
            CountControllers = client.CountControllers;
            License = client.Integration.ReadyDecicionNumberNavigation.DecicionName;
            foreach (MainInfo mainInfo in client.MainInfos)
            {
                TicketList.Add(new TicketListTemplate(mainInfo.AppealNumber.ToString(),
                                                      client.Integration.ReadyDecicionNumberNavigation.DecicionName,
                                                      mainInfo.AppealDate.ToString(),
                                                      client.CompanyName,
                                                      mainInfo.Email,
                                                      mainInfo.ContactNumber,
                                                      mainInfo.StatusNumberNavigation.Condition,
                                                      ""));
            }
            if (client.Integration.Awt)       ModuleList.Add(new ClientParamTemplate("", "  "));
            if (client.Integration.Gdp)       ModuleList.Add(new ClientParamTemplate("", "  "));
            if (client.Integration.Etsc)      ModuleList.Add(new ClientParamTemplate("", "   "));
            if (client.Integration.Onec)      ModuleList.Add(new ClientParamTemplate("", "   1"));
            if (client.Integration.Cp)        ModuleList.Add(new ClientParamTemplate("", ""));
            if (client.Integration.Roe)       ModuleList.Add(new ClientParamTemplate("", "  "));
            if (client.Integration.Rod)       ModuleList.Add(new ClientParamTemplate("", " "));
            if (client.Integration.Ps)        ModuleList.Add(new ClientParamTemplate("", " "));
            if (client.Integration.Sod)       ModuleList.Add(new ClientParamTemplate("", " "));
            if (client.Integration.Pa)        ModuleList.Add(new ClientParamTemplate("", " "));
            if (client.Integration.MtOffline) ModuleList.Add(new ClientParamTemplate("", "  (Offline)"));
        }
        public string CompanyName { get; set; }
        public int CountTickets { get; set; }
        public int CountControllers { get; set; }
        public string License { get; set; }
        public ObservableCollection<TicketListTemplate> TicketList { get; set; } = [];
        public ObservableCollection<ClientParamTemplate> ModuleList { get; set; } = [];
    }
}