using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using ReactiveUI;
using LiveChartsCore;
using LiveChartsCore.SkiaSharpView;
using LiveChartsCore.SkiaSharpView.Painting;
using SkiaSharp;
using Practice.Models;
using System.Linq;

namespace Practice.ViewModels
{
	public class MainViewModel : ReactiveObject
    {
        public MainViewModel(Staff currentUser) 
        {
            CurrentUser = currentUser;
            SigurNewsContext = new SigurNewsContext();
            List<News> DBNews = [.. SigurNewsContext.News];

            foreach (News item in DBNews) 
            {
                News.Add(new NewsItemTemplate(item.Heading, item.Body));
            }
            var Tickets = DBManager.TicketsByDates(CurrentUser.StaffLogin);
            foreach (var item in Tickets) 
            {
                TicketsByDay.Add(new TicketsCount(item.AppealDate.ToString(), item.TotalAppeals, item.SuccessfulAppeals));
            }
            Tickets = DBManager.TicketsByMonth(CurrentUser.StaffLogin);
            foreach (var item in Tickets)
            {
                int month = item.Month;
                DateTime date = new(DateTime.Now.Year, month, 1); // 1-  
                TicketsByMonth.Add(new TicketsCount(date.ToString("MMMM"), item.TotalAppeals, item.SuccessfulAppeals));
            }
        }

        public Staff CurrentUser { get; set; }
        private SigurNewsContext SigurNewsContext { get; set; }

        public ObservableCollection<NewsItemTemplate> News { get; set; } = [];

        public ObservableCollection<TicketsCount> TicketsByDay { get; } = [];

        public ObservableCollection<TicketsCount> TicketsByMonth { get; } = [];
    }

    public class NewsItemTemplate(string header, string content)
    {
        public string Header { get; } = header;
        public string Content { get; } = content;
    }

    public class TicketsCount
    {
        public TicketsCount(string header, double all, double success)
        {
            Header = header;
            All = [all];
            Success = [success];
            Series =
            [
                new PieSeries<double> { Values = Success, Fill = new SolidColorPaint(new SKColor(242, 81, 27)), InnerRadius = 20},
                new PieSeries<double> { Values = [All[0] - Success[0]], Fill = new SolidColorPaint(new SKColor(255, 255, 255)), InnerRadius = 20},
            ];
        }
        public string Header { get; }
        public double[] All { get; }
        public double[] Success { get; }
        public ISeries[] Series { get; set; }
    }
}