﻿using Microsoft.EntityFrameworkCore;
using Practice.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using ReactiveUI;
using System.Collections.ObjectModel;

namespace Practice.ViewModels
{
    public class MainWindowViewModel : ViewModelBase
    {
#pragma warning disable CS8618 // Поле, не допускающее значения NULL, должно содержать значение, отличное от NULL, при выходе из конструктора. Возможно, стоит объявить поле как допускающее значения NULL.
        public MainWindowViewModel(Staff currentUser)
#pragma warning restore CS8618 // Поле, не допускающее значения NULL, должно содержать значение, отличное от NULL, при выходе из конструктора. Возможно, стоит объявить поле как допускающее значения NULL.
        {
            //string login = "Ivanov_Ivan";
            //string password = "password_Ivan";
            CurrentUser = currentUser;
            MenuItems =
            [
                new ListItemTemplate(new MainViewModel(CurrentUser), "ГЛАВНАЯ"),
                new ListItemTemplate(new TicketViewModel(CurrentUser), "ТИКЕТЫ"),
                new ListItemTemplate(new MyTicketViewModel(CurrentUser), "МОИ ТИКЕТЫ"),
                new ListItemTemplate(new ClientViewModel(CurrentUser), "КЛИЕНТЫ")
            ];
            SelectedListItem = MenuItems[0];
        }

        public Staff? CurrentUser { get; set; }
        
        private ReactiveObject currentPage;
        public ReactiveObject CurrentPage { get => currentPage; set => this.RaiseAndSetIfChanged(ref currentPage, value); }

        private ListItemTemplate? selectedListItem;
        public ListItemTemplate? SelectedListItem { get => selectedListItem; set => OnSelectedListItemChanged(ref selectedListItem, value); }

        private void OnSelectedListItemChanged(ref ListItemTemplate? backingField, ListItemTemplate? value)
        {
            if (value == null) return;
            CurrentPage = value.ViewModel;
            backingField = value;
            if (value.ViewModel is MyTicketViewModel)
            {
                MyTicketViewModel ViewModel = (MyTicketViewModel)value.ViewModel;
                ViewModel.RenewTicketList();
            }
        }

        public ObservableCollection<ListItemTemplate> MenuItems { get; set; }
    }

    public class ListItemTemplate(ReactiveObject viewModel, string name)
    {
        public string Label { get; } = name;
        public ReactiveObject ViewModel { get; } = viewModel;
    }

    public static class DBManager
    {
        public static SigurDatabaseContext DatabaseContext { get; set; } = new SigurDatabaseContext();

        public static dynamic TicketsByDates(string login)
        {
#pragma warning disable CS8602 // Разыменование вероятной пустой ссылки.
            var appealsByStaff = DatabaseContext.MainInfos
                .Include(mi => mi.StatusNumberNavigation)
                .Join(DatabaseContext.AppealsTakens,
                    mi => mi.AppealNumber,
                    at => at.AppealNumber,
                    (mi, at) => new { mi, at })
                .Join(DatabaseContext.Staff,
                    mat => mat.at.StaffLogin,
                    st => st.StaffLogin,
                    (mat, st) => new { mat.mi, mat.at, st })
                .Where(x => x.st.StaffLogin == login)
                .GroupBy(x => x.mi.AppealDate)
                .Select(g => new
                {
                    AppealDate = g.Key,
                    TotalAppeals = g.Count(),
                    SuccessfulAppeals = g.Count(x => x.mi.StatusNumberNavigation.Condition == "Решено")
                })
                .OrderByDescending(g => g.AppealDate)
                .ToList();
#pragma warning restore CS8602 // Разыменование вероятной пустой ссылки.
            return appealsByStaff;
        }

        public static dynamic TicketsByMonth(string login)
        {
#pragma warning disable CS8602 // Разыменование вероятной пустой ссылки.
            var appealsByStaffByMonth = DatabaseContext.MainInfos
                .Include(mi => mi.StatusNumberNavigation)
                .Join(DatabaseContext.AppealsTakens,
                    mi => mi.AppealNumber,
                    at => at.AppealNumber,
                    (mi, at) => new { mi, at })
                .Join(DatabaseContext.Staff,
                    mat => mat.at.StaffLogin,
                    st => st.StaffLogin,
                    (mat, st) => new { mat.mi, mat.at, st })
                .Where(x => x.st.StaffLogin == login)
                .GroupBy(x => x.mi.AppealDate.Month) // Группировка только по месяцу
                .Select(g => new
                {
                    Month = g.Key,
                    TotalAppeals = g.Count(),
                    SuccessfulAppeals = g.Count(x => x.mi.StatusNumberNavigation.Condition == "Решено")
                })
                .OrderByDescending(g => g.Month) // Сортировка по месяцу
                .ToList();
#pragma warning restore CS8602 // Разыменование вероятной пустой ссылки.
            return appealsByStaffByMonth;
        }

        public static List<StatusInfo> StatusesList
        {
            get
            {
                List<StatusInfo> Statuses = [.. DatabaseContext.StatusInfos];
                return Statuses;
            }
        }



        public static void AddAppealToTaken(int appealNumber, string staffLogin)
        {
            // 1. Найти заявку по номеру
            var appeal = DatabaseContext.MainInfos.FirstOrDefault(a => a.AppealNumber == appealNumber);

            // 2. Найти статус "В работе"
            var statusInWork = DatabaseContext.StatusInfos.FirstOrDefault(s => s.Condition == "В работе");

            // 3. Установить статус заявки "В работе"
            appeal.StatusNumber = statusInWork.StatusNumber;
            DatabaseContext.SaveChanges();

            // 4. Добавить запись в таблицу взятых заявок
            var appealsTaken = new AppealsTaken
            {
                StaffLogin = staffLogin,
                AppealNumber = appealNumber,
                AppealNumberNavigation = appeal
            };

            DatabaseContext.AppealsTakens.Add(appealsTaken);
            DatabaseContext.SaveChanges();
        }

        public static List<MainInfo> GetAppeals(
            string? baseModuleName = null, // Название типа базового модуля
            string? statusCondition = null, // Название статуса заявки
            DateOnly? startDate = null, // Первая дата (включительно)
            DateOnly? endDate = null // Вторая дата (включительно)
        )
        {
            IQueryable<MainInfo> query = DatabaseContext.MainInfos
                .Include(mi => mi.AppealReasonNavigation)
                .Include(mi => mi.CompanyNameNavigation)
                .ThenInclude(ci => ci.Integration)
                .ThenInclude(i => i.ReadyDecicionNumberNavigation)
                .Include(mi => mi.StatusNumberNavigation)
                .OrderByDescending(mi => mi.AppealDate);

            // Фильтр по типу базового модуля
            if (!string.IsNullOrEmpty(baseModuleName) && baseModuleName != "-")
            {
                query = query.Where(mi =>
                    mi.CompanyNameNavigation != null &&
                    mi.CompanyNameNavigation.Integration != null &&
                    mi.CompanyNameNavigation.Integration.ReadyDecicionNumberNavigation != null &&
                    mi.CompanyNameNavigation.Integration.ReadyDecicionNumberNavigation.DecicionName == baseModuleName);
            }

            // Фильтр по статусу заявки
            if (!string.IsNullOrEmpty(statusCondition) && statusCondition != "-")
            {
                query = query.Where(mi => mi.StatusNumberNavigation != null &&
                                          mi.StatusNumberNavigation.Condition == statusCondition);
            }

            // Фильтр по датам
            if (startDate != null)
            {
                query = query.Where(mi => mi.AppealDate >= startDate);
            }

            if (endDate != null)
            {
                query = query.Where(mi => mi.AppealDate <= endDate);
            }

            // Выполняем запрос и возвращаем список
            return [.. query];
        }

        public static List<string> GetBaseModuleNames()
        {
            return [.. DatabaseContext.ReadyDecicions.Select(bm => bm.DecicionName)];
        }
        public static List<AppealData> GetAppealsByStaff(string staffLogin, bool onlyInWork = false)
        {
#pragma warning disable CS8602 // Разыменование вероятной пустой ссылки.
#pragma warning disable CS8604 // Возможно, аргумент-ссылка, допускающий значение NULL.
            var query = DatabaseContext.AppealsTakens
                .Where(at => at.StaffLogin == staffLogin);

            if (onlyInWork)
            {
                query = query.Where(at => at.AppealNumberNavigation.StatusNumberNavigation.Condition == "В работе");
            }

            var select_query = query.Select(at => new AppealData
                (
                    at.AppealNumber.ToString(),
                    at.AppealNumberNavigation.CompanyNameNavigation.CompanyName,
                    at.AppealNumberNavigation.Fcs,
                    at.AppealNumberNavigation.ContactNumber,
                    at.AppealNumberNavigation.Email,
                    at.AppealNumberNavigation.AppealDate,
                    at.AppealNumberNavigation.CompanyNameNavigation.Integration.ReadyDecicionNumberNavigation.DecicionName,
                    at.AppealNumberNavigation.StatusNumberNavigation.Condition
                ));

            return [.. select_query];
#pragma warning restore CS8604 // Возможно, аргумент-ссылка, допускающий значение NULL.
#pragma warning restore CS8602 // Разыменование вероятной пустой ссылки.
        }

        public static List<string> GetAppealProblemAndSolution(int appealNumber)
        {
            var complitedAppeal = DatabaseContext.ComplitedAppeals
                .FirstOrDefault(ca => ca.NumberAppeal == appealNumber);

            if (complitedAppeal != null)
            {
                return
            [
                complitedAppeal.AppealProblem,
                complitedAppeal.AppealDecision
            ];
            }
            else
            {
                return ["", ""];
            }
        }

        public static void SetAppealStatus(int appealNumber, string newStatus, string appealProblem, string appealDecision)
        {
            // 1. Найти заявку по номеру
            var appeal = DatabaseContext.MainInfos.FirstOrDefault(a => a.AppealNumber == appealNumber);

            // 2. Найти статус
            var status = DatabaseContext.StatusInfos.FirstOrDefault(s => s.Condition == newStatus);

            // 3. Установить новый статус заявки
            appeal.StatusNumber = status.StatusNumber;
            DatabaseContext.SaveChanges();

            // 4. Если новый статус == "Решено", то добавить заявку в таблицу выполненых
            if (newStatus == "Решено")
            {
                var complitedAppeal = new ComplitedAppeal
                {
                    NumberAppeal = appealNumber,
                    AppealProblem = appealProblem,
                    AppealDecision = appealDecision,
                    NumberAppealNavigation = appeal
                };

                DatabaseContext.ComplitedAppeals.Add(complitedAppeal);
                DatabaseContext.SaveChanges();
            }
        }

        public static List<CompanyInfo> GetCompanies()
        {
            var companies = DatabaseContext.CompanyInfos
                .Include(c => c.Integration)
                    .ThenInclude(i => i.ReadyDecicionNumberNavigation)
                .Include(c => c.Integration)
                    .ThenInclude(i => i.VariantBaseModuleNavigation)
                .Include(c => c.MainInfos)
                    .ThenInclude(mi => mi.AppealReasonNavigation)
                .Include(c => c.MainInfos)
                    .ThenInclude(mi => mi.StatusNumberNavigation)
                .ToList();
            return companies;
        }
    }

    public class AppealData(string number, string companyName, string contactPerson, string phone, string email, DateOnly appealDate, string readyDecisionName, string status)
    {
        public string Number { get; set; } = number;
        public string CompanyName { get; set; } = companyName;
        public string ContactPerson { get; set; } = contactPerson;
        public string Phone { get; set; } = phone;
        public string Email { get; set; } = email;
        public DateOnly AppealDate { get; set; } = appealDate;
        public string ReadyDecisionName { get; set; } = readyDecisionName;
        public string Status { get; set; } = status;
    }
}
