using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Microsoft.EntityFrameworkCore;
using Practice.Models;
using ReactiveUI;

namespace Practice.ViewModels
{
	public class MyTicketViewModel : ReactiveObject
    {
        public MyTicketViewModel(Staff currentUser)
        {
            CurrentUser = currentUser;
            var Statuses = DBManager.StatusesList;
            foreach (var stat in Statuses) 
            {
                StatusList.Add(new StatusItemsTemplate(stat.Condition));
            }
            RenewTicketList();
        }
        private string ticketNumber;
        private string problem;
        private string solution;
        private bool isActive;
        private bool showActive = false;
        private ObservableCollection<MyTicketListTemplate> ticketList = [];
        private ObservableCollection<ClientParamTemplate> paramList;
        private StatusItemsTemplate selectedStatus;
        private MyTicketListTemplate selectedTicket;

        public Staff CurrentUser { get; set; }
        public string TicketNumber { get => ticketNumber; set => this.RaiseAndSetIfChanged(ref ticketNumber, value); }
        public string Problem { get => problem; set => this.RaiseAndSetIfChanged(ref problem, value); }
        public string Solution { get => solution; set => this.RaiseAndSetIfChanged(ref solution, value); }
        public bool IsActive { get => isActive; set => this.RaiseAndSetIfChanged(ref isActive, value); }
        public bool ShowActive { get => showActive; set => OnCheckBoxChange(ref showActive, value); }
        public ObservableCollection<StatusItemsTemplate> StatusList { get; set; } = [];
        public ObservableCollection<MyTicketListTemplate> TicketList { get => ticketList; set => this.RaiseAndSetIfChanged(ref ticketList, value); }
        public ObservableCollection<ClientParamTemplate> ParamList { get => paramList; set => this.RaiseAndSetIfChanged(ref paramList, value); }
        public StatusItemsTemplate SelectedStatus { get => selectedStatus; set => this.RaiseAndSetIfChanged(ref selectedStatus, value); }
        public MyTicketListTemplate SelectedTicket { get => selectedTicket; set => OnSelectedTicketChange(ref selectedTicket, value); }

        private void OnCheckBoxChange(ref bool backingField, bool value) 
        {
            backingField = value;
            RenewTicketList();
        }

        private void OnSelectedTicketChange(ref MyTicketListTemplate backingField, MyTicketListTemplate value)
        {
            backingField = value;
            if (value == null) return;
            SelectedStatus = new StatusItemsTemplate(value.Status);
            TicketNumber = $" {value.Number}";
            List<string> Desc = DBManager.GetAppealProblemAndSolution(Convert.ToInt32(value.Number));

            Problem = Desc[0];
            Solution = Desc[1];
            IsActive = value.Status == " ";

            ParamList = [
                new ClientParamTemplate("", value.CompanyName),
                new ClientParamTemplate(" ", value.ContactPerson),
                new ClientParamTemplate(" ", value.Phone),
                new ClientParamTemplate("", value.Email),
                new ClientParamTemplate("", value.AppealDate),
                new ClientParamTemplate(" ", value.ReadyDecisionName),
                ];
        }

        public void SaveChanges() 
        {
            DBManager.SetAppealStatus(Convert.ToInt32(SelectedTicket.Number), SelectedStatus.Value, Problem, Solution);
            IsActive = SelectedStatus.Value == " ";
            RenewTicketList();
        }

        public void RenewTicketList() 
        {
            var Tickets = DBManager.GetAppealsByStaff(CurrentUser.StaffLogin, ShowActive);
            TicketList = [];
            foreach (var ticket in Tickets)
            {
                TicketList.Add(new MyTicketListTemplate(ticket.Number,
                                                        ticket.Status,
                                                        ticket.CompanyName,
                                                        ticket.ContactPerson,
                                                        ticket.Phone,
                                                        ticket.Email,
                                                        ticket.AppealDate.ToString(),
                                                        ticket.ReadyDecisionName));
            }
            if (TicketList.Count > 0) SelectedTicket = TicketList[0];
        }
    }

    public class MyTicketListTemplate
    {
        public MyTicketListTemplate(string number, string status, string companyName, string contactPerson, string phone, string email, string appealDate, string readyDecisionName)
        {
            string icon = "";
            Number = number;
            Status = status;
            switch (Status)
            {
                case " ":
                    {
                        icon = "/Assets/Polygon 1.svg";
                        break;
                    }
                case "":
                    {
                        icon = "/Assets/Polygon 2.svg";
                        break;
                    }
                case " ":
                    {
                        icon = "/Assets/Polygon 3.svg";
                        break;
                    }
                default:
                    {
                        break;
                    }
            }
            IconImage = icon;
            CompanyName = companyName;
            ContactPerson = contactPerson;
            Phone = phone;
            Email = email;
            AppealDate = appealDate;
            ReadyDecisionName = readyDecisionName;
        }

        public string Number { get; }
        public string Status { get; }
        public string IconImage { get; set; }

        public string CompanyName { get; set; }
        public string ContactPerson { get; set; }
        public string Phone { get; set; }
        public string Email { get; set; }
        public string AppealDate { get; set; }
        public string ReadyDecisionName { get; set; }
    }

    public class ClientParamTemplate(string paramName, string paramValue)
    {
        public string ParamName { get; } = paramName;
        public string ParamValue { get; } = paramValue;
    }

    public class StatusItemsTemplate(string value)
    {
        public string Value { get; set; } = value;
    }
}