using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Practice.Models;
using ReactiveUI;

namespace Practice.ViewModels
{
	public class TicketViewModel : ReactiveObject
    {
        private FilterItemsTemplate moduleFilter;
        private FilterItemsTemplate statusFilter;
        private DateTime firstDate = new DateTime(1950, 1, 1);
        private DateTime secondDate = DateTime.Today;
        private ObservableCollection<TicketListTemplate> ticketList = [];

        public TicketViewModel(Staff currentUser) 
        {
            CurrentUser = currentUser;
            List<StatusInfo> Statuses = DBManager.StatusesList;
            foreach (StatusInfo stat in Statuses)
            {
                StatusFilters.Add(new FilterItemsTemplate(stat.Condition));
            }

            List<string> ReadyDecicions = DBManager.GetBaseModuleNames();
            foreach (string s in ReadyDecicions) 
            {
                ModuleFilters.Add(new FilterItemsTemplate(s));
            }

            SetTickets();
        }

        public Staff CurrentUser { get; set; }

        public ObservableCollection<FilterItemsTemplate> ModuleFilters { get; set; } = [new FilterItemsTemplate("-")];
        public ObservableCollection<FilterItemsTemplate> StatusFilters { get; set; } = [new FilterItemsTemplate("-")];

        public FilterItemsTemplate ModuleFilter { get => moduleFilter; set => OnFilterChange(ref moduleFilter, value); }
        public FilterItemsTemplate StatusFilter { get => statusFilter; set => OnFilterChange(ref statusFilter, value); }

        public DateTime FirstDate { get => firstDate; set => OnFilterDateChange(ref firstDate, value); }
        public DateTime SecondDate { get => secondDate; set => OnFilterDateChange(ref secondDate, value); }

        public ObservableCollection<TicketListTemplate> TicketList { get => ticketList; set => this.RaiseAndSetIfChanged(ref ticketList, value); }
        private void SetTickets()
        {
            ObservableCollection<TicketListTemplate> ticketList = [];
            var Tickets = DBManager.GetAppeals(ModuleFilter?.Value ?? null, StatusFilter?.Value ?? null, DateOnly.FromDateTime(FirstDate), DateOnly.FromDateTime(SecondDate));
            foreach (var ticket in Tickets)
            {
                ticketList.Add(new TicketListTemplate(ticket.AppealNumber.ToString(),
                                                      ticket.CompanyNameNavigation.Integration.ReadyDecicionNumberNavigation.DecicionName,
                                                      ticket.AppealDate.ToString(),
                                                      ticket.CompanyName,
                                                      ticket.Email,
                                                      ticket.ContactNumber,
                                                      ticket.StatusNumberNavigation.Condition,
                                                      CurrentUser.StaffLogin));
            }
            TicketList = ticketList;
        }

        private void OnFilterDateChange(ref DateTime filter, DateTime newValue) 
        { 
            filter = newValue;
            SetTickets();
        }

        private void OnFilterChange(ref FilterItemsTemplate filter, FilterItemsTemplate newValue)
        {
            filter = newValue;
            SetTickets();
        }
    }

	public class FilterItemsTemplate(string value)
    {
        public string Value { get; set; } = value;
    }

    public class TicketListTemplate(string number, string type, string date, string client, string email, string phone, string status, string currentUser) : ReactiveObject
    {
        private bool isActive = status == "";

        public string Number { get; } = number;
        public string Type { get; } = type;
        public string Date { get; } = date;
        public string Client { get; } = client;
        public string Email { get; } = email;
        public string Phone { get; } = phone;
        public string Status { get; } = status;
        public string CurrentUser { get; } = currentUser;
        public bool IsActive { get => isActive; set => this.RaiseAndSetIfChanged(ref isActive, value); }
        public void TakeTicket() 
        {
            DBManager.AddAppealToTaken(Convert.ToInt32(Number), CurrentUser);
            IsActive = false;
        }
    }
}